#include <GDD/dag.h>
#include <assert.h>

int main(int argc, char *argv[]) {
  // two operations 
  GDD_Opcode *add_op =  GDD_opcode_create("add", 2, 0);
  GDD_opcode_extend_delta_w(add_op, "GR", 1);

  GDD_Opcode *load_op = GDD_opcode_create("load", 3, 1);
  GDD_opcode_extend_delta_w(load_op, "GR", 1);

  // build the architecture description
  GDD_Arch *arch = GDD_arch_create();
  GDD_arch_add_regtype(arch, "GR", 32);
  GDD_arch_add_opcode(arch, add_op);
  GDD_arch_add_opcode(arch, load_op);

  // build a GDD
  SCEDA_Graph *g = SCEDA_graph_create((SCEDA_delete_fun)GDD_node_label_delete, (SCEDA_delete_fun)GDD_edge_label_delete); 

  SCEDA_Vertex *v0 = SCEDA_graph_add_vertex(g, GDD_node_label_create(1, load_op));
  SCEDA_Vertex *v1 = SCEDA_graph_add_vertex(g, GDD_node_label_create(2, load_op));
  SCEDA_Vertex *v2 = SCEDA_graph_add_vertex(g, GDD_node_label_create(3, add_op));

  SCEDA_graph_add_edge(g, v0, v2, GDD_edge_label_create("GR", "flowdep_reg", 5, 0));
  SCEDA_graph_add_edge(g, v1, v2, GDD_edge_label_create("GR", "flowdep_reg", 5, 0));

  // build the DDAG
  GDD_DAG *ddg = GDD_dag_create(g);
  if(ddg != NULL) {
    if(GDD_dag_lt(ddg, v0, v1)) {
      fprintf(stdout, "v0 < v1\n");
    } else if(GDD_dag_lt(ddg, v1, v0)) {
      fprintf(stdout, "v0 > v1\n");
    } else {
      assert(GDD_dag_parallel(ddg, v0, v1));
      fprintf(stdout, "v0 # v1\n");      
    }
    GDD_dag_delete(ddg);
  }

  SCEDA_graph_delete(g); // delete g
  GDD_arch_delete(arch);     // and the architecture

  return 0;
}

