#include <stdio.h>
#include <libxml/parser.h>

#include <GDD/arch_xml.h>
#include <GDD/ddg_xml.h>
#include <GDD/ddg.h>
#include <SIRA/reduce.h>

#include <assert.h>

int main(int argc, char *argv[]) {
  // initialise XML parser
  xmlInitParser();

  // read architecture
  GDD_Arch *arch = GDD_arch_of_xml_file("arch.xml");
  assert(arch != NULL);
      
  // read ddg (ignore interloop dependency)
  SCEDA_Graph *dag = GDD_of_xml_file(TRUE, arch, "ddg.xml",NULL);
  assert(dag != NULL);

  // clean up XML parser
  xmlCleanupParser();

  // initialise settings for the solver
  SIRA_WeightMap *weight_map = SIRA_weight_map_create();
  SIRA_weight_map_set_weight_of_type(weight_map, "GR", 1); // reduce GR saturation

  SCEDA_List *added_edges;
  if(SIRA_reduce(dag, weight_map, &added_edges) >= 0) {
    fprintf(stdout,"success\n");
    fprintf(stdout,"%d edges added\n", SCEDA_list_size(added_edges));

    // remove added edges
    GDD_delete_edges(dag, added_edges);
    // clear the list
    SCEDA_list_delete(added_edges);
  }

  SIRA_weight_map_delete(weight_map);

  // delete the graph and the architecture
  SCEDA_graph_delete(dag);
  GDD_arch_delete(arch);

  return 0;
}
