/*
   Copyright Universite de Versailles Saint-Quentin en Yvelines 2009
   AUTHORS: Sebastien Briais, Sid Touati

   This file is part of SIRA.
   
   SIRA is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.
   
   SIRA is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with SIRA.  If not, see
   <http://www.gnu.org/licenses/>.
*/
/* Include string match and hash functions */
#include "stringutil.h"

#include "weightmap.h"
#include "boxed.h"

/** Create an empty weight map.

    @return a new weight map
*/
SIRA_WeightMap *SIRA_weight_map_create() {
  return SCEDA_hashmap_create((SCEDA_delete_fun)free, (SCEDA_delete_fun)boxed_delete, (SCEDA_match_fun)string_match, (SCEDA_hash_fun)string_hash);
}

/** Set weight of given type.

    @param[in] weight_map = weight map
    @param[in] type = type
    @param[in] weight = new weight

    @return 0 in case of success, <>0 otherwise */
int SIRA_weight_map_set_weight_of_type(SIRA_WeightMap *weight_map, const char *type, int weight) {
  if(weight < 0) {
    return -1;
  }
  if(!SCEDA_hashmap_contains_key(weight_map, type)) {
    return SCEDA_hashmap_put(weight_map, safe_strdup(type), boxed_create(int, weight), NULL);
  }
  return -1;
}

/** Return the next element in the weight map iterator

    @param[in] iter = iterator
    @param[out] type = type

    @return the weight of the type
    (will fail if iterator is empty) */
int SIRA_weight_map_iterator_next(SIRA_WeightMapIterator *iter, char **type) {
  boxed(int) w = SCEDA_hashmap_iterator_next(iter, type);
  return boxed_get(w);
}

/** Return weight of given type in the weight map.

    @param[in] weight_map = weight map
    @param[in] type = wanted type

    @return weight of type or -1 if undefined */
int SIRA_weight_map_get_weight_of_type(SIRA_WeightMap *weight_map, const char *type) {
  boxed(int) w = SCEDA_hashmap_get(weight_map, type);
  if(w == NULL) {
    return -1;
  }
  return boxed_get(w);
}
